
C     + ---------------------------------------------------------- +
         Subroutine StereoSouth (E,N,GEddxx,lon,lat)
C     | Compute the lon, lat from Oblique Stereographic Projection |
C     | Written by Cecile Agosta                          17-05-10 |
C     | EPSG Polar Stereographic transformation Variant B          |
C     | (http://www.epsg.org/guides/docs/G7-2.pdf)                 |
C     | Equivalent to EPSG 3031 (WGS-84 ellipsoid)                 |
C     + ---------------------------------------------------------- +
C     |                                                            |
C     | INPUT :  E      : Stereo coordinate on the East  (X, km)   |
C     | ^^^^^^^  N      : Stereo coordinate on the North (Y, km)   |
C     |          GEddxx : Longitude of X axis (90=East, clockwise) |
C     |         [lat true = 71 S]                                  |
C     |                                                            |
C     | OUTPUT : lon    : longitude (deg)                          |
C     | ^^^^^^^  lat    : latitude  (deg)                          |
C     |                                                            |
C     + ---------------------------------------------------------- +
      IMPLICIT NONE
     
      INCLUDE 'MARdim.inc'
     
C      + General Variables
C      + -----------------
      Real,INTENT(in ) :: E,N,GEddxx
      Real,INTENT(out) :: lon,lat
      
C      + Local Variables
C      + ---------------
      Real ddista
      
C      + Constants
C      + ---------
      Real  aa,ex,pi,degrad,latF,FE,FN,tF,mF,k0,t,rho,khi,lon0
      Real  trulat
     
C      + aa : (km) = demi grand axe
         aa     = 6378.1370
C      + ex : excentricity WGS-84
       ex     = 0.081819190842621
C      + trulat : Latitude of standard parallel, 71 S for ESPG 3031
         trulat = -71.
      pi     = 4. * atan(1.)
      degrad = pi / 180.
     
      latF = trulat*degrad
      lon0 = (GEddxx-90.)*degrad
     
      FE = 0. !False Easting
      FN = 0. !False Northing
     
C      + Polar Stereographic Projection
C      + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     
      tF = tan (pi/4 + latF/2) /
     .    ( (1 + ex*sin(latF)) / (1 - ex*sin(latF)) )**(ex/2)
      mF = cos(latF) / (1 - ex**2 * sin(latF)**2)**0.5
      k0 = mF   *( (1+ex)**(1+ex) * (1-ex)**(1-ex) )**0.5 / (2*tF)
     
      rho = ( (E-FE)**2 + (N-FN)**2 )**0.5
      t   = rho * ( (1+ex)**(1+ex) * (1-ex)**(1-ex) )**0.5 / (2*aa*k0)
      khi = 2*atan(t) - pi/2
     
      lat = khi
     .      + (  ex**2/2   +  5*ex**4/24  +     ex**6/12 + 13*ex**8/360)
     .        *sin(2*khi)
     .      + (7*ex**4/48  + 29*ex**6/240 + 811*ex**8/11520            )
     .        *sin(4*khi)
     .      + (7*ex**6/120 + 81*ex**8/1120                             )
     .        *sin(6*khi)
     .      + (            4279*ex**8/161280                           )
     .        *sin(8*khi)
     
      if      (E-FE .eq. 0. .and. N-FN .ge. 0) then
        lon = lon0
      else if (E-FE .eq. 0. .and. N-FN .le. 0) then
        lon = lon0 + pi
      else
        lon = lon0 + atan2(E-FE,N-FN)
      endif
           
      lat = lat / degrad
      lon = lon / degrad
     
      Return
      END Subroutine StereoSouth
     
     
C     + ---------------------------------------------------------- +
      Subroutine StereoSouth_inverse (lon,lat,lonE,E,N)
C     | Compute Oblique Stereographic Projection from lon,lat      |
C     | Written by Cecile Agosta                          17-05-10 |
C     | EPSG Polar Stereographic transformation Variant B          |
C     | (http://www.epsg.org/guides/docs/G7-2.pdf)                 |
C     | Equivalent to EPSG 3031 (WGS-84 ellipsoid)                 |
C     + ---------------------------------------------------------- +
C     |                                                            |
C     | INPUT : lon     : Longitude (deg)                          |
C     | ^^^^^^^ lat     : Latitude  (deg)                          |
C     |        lon0    : Longitude of X axis (90=East, clockwise)  |
C     |       [lat true = 71 S]                                    |
C     |                                                            |
C     | OUTPUT : E      : Stereo coordinate on the East  (X, km)   |
C     | ^^^^^^^^ N      : Stereo coordinate on the North (Y, km)   |
C     |                                                            |
C     + ---------------------------------------------------------- +
      IMPLICIT NONE
     
      INCLUDE 'MARdim.inc'
     
C      + General Variables
C      + -----------------
      Real,INTENT(in ) :: lon,lat,lonE
      Real,INTENT(out) :: E,N
     
C      + Local Variables
C      + ---------------
      Real  costru,ddista
     
C      + Constants
C      + ---------
      Real  aa,ex,pi,degrad,latF,FE,FN,tF,mF,k0,t,rho,lonrad,latrad
      Real  lon0,trulat
     
C      + aa : (km) = demi grand axe
         aa     = 6378.1370
C      + ex : excentricity WGS-84
         ex     = 0.081819190842621
C      + trulat : Latitude of standard parallel, 71 S for ESPG 3031
         trulat = -71.
      pi     = 4. * atan(1.)
      degrad = pi / 180.
     
      latF   = trulat*degrad
      lon0   = (lonE-90)*degrad
      lonrad = lon *degrad
      latrad = lat *degrad
     
C     + False Easting
      FE = 0.
C     + False Northing
      FN = 0. 
     
C      + Polar Stereographic Projection
C      + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     
      tF  = tan (pi/4 + latF  /2) /
     .      ( (1 + ex*sin(latF  )) / (1 - ex*sin(latF  )) )**(ex/2)
      mF  = cos(latF) / (1 - ex**2 * sin(latF)**2)**0.5
      k0  = mF*( (1+ex)**(1+ex) * (1-ex)**(1-ex) )**0.5 / (2*tF)
     
      t   = tan (pi/4 + latrad/2) /
     .      ( (1 + ex*sin(latrad)) / (1 - ex*sin(latrad)) )**(ex/2)
      rho = 2*aa*k0*t / ( (1+ex)**(1+ex) * (1-ex)**(1-ex) )**0.5
     
      E = FE + rho*sin (lonrad - lon0)
      N = FN + rho*cos (lonrad - lon0)
     
      RETURN
      END Subroutine StereoSouth_inverse
